;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  MSHACL.lisp
;;  Copyright (c) 1995, GRAME.  All rights reserved.
;;
;;  This file contains definitions for records and Pascal style routines, 
;;  used for interfacing ACL with MidiShare 1.52, real-time multitasking 
;;  Midi operating system.
;;
;;  Description:  MidiShare interface to Allegro Common Lisp
;;  Date:        30-11-95
;;; --------------------------------------------------------------------

(defpackage :MIDISHARE
  (:use "COMMON-LISP" "ALLEGRO" "C-TYPES")
  (:export "typeNote" "typeKeyOn" "typeKeyOff" "typeKeyPress" 
     "typeCtrlChange" "typeProgChange" 
            "typeChanPress" "typePitchWheel" "typeSongPos" "typeSongSel" 
     "typeClock" "typeStart"
            "typeContinue" "typeStop" "typeTune" "typeActiveSens" "typeReset" 
   "typeSysEx" "typeStream"
            "typePrivate" "typeProcess" "typeDProcess" "typeQFrame" 
"typeReserved" "typeDead"
            "typeCtrl14b" "typeNonRegParam" "typeRegParam" "typeSeqNum" 
"typeText" "typeCopyright" 
            "typeSeqName" "typeInstrName" "typeLyric" "typeMarker" 
"typeCuePoint" "typeChanPrefix" 
            "typeEndTrack" "typeTempo" "typeSMPTEOffset" "typeTimeSign" 
"typeKeySign" "typeSpecific"
            
            "MIDIerrSpace" "MIDIerrRefNum" "MIDIerrBadType" 
"MIDIerrIndex"
            "ModemPort" "PrinterPort"           
            "MIDIOpenAppl" "MIDICloseAppl" "MIDIChgName" 
"MIDIChgConnect" "MIDIOpenModem" "MIDICloseModem"
            "MIDIOpenPrinter" "MIDIClosePrinter"            
           "MidiGetVersion" "MidiCountAppls" "MidiGetIndAppl" 
"MidiGetNamedAppl" "MidiOpen" "MidiClose"
           "MidiGetName" "MidiSetName" "MidiGetInfo" "MidiSetInfo" 
"MidiGetFilter" "MidiSetFilter"
            "MidiGetRcvAlarm" "MidiSetRcvAlarm" "MidiGetApplAlarm" 
"MidiSetApplAlarm" "MidiConnect"
            "MidiIsConnected" "MidiGetPortState" "MidiSetPortState" 
"MidiFreeSpace" "MidiNewEv" "MidiCopyEv"
            "MidiFreeEv" "MidiSetField" "MidiGetField" "MidiAddField" 
"MidiCountFields" "MidiNewSeq"
            "MidiAddSeq" "MidiFreeSeq" "MidiClearSeq" "MidiApplySeq" 
"MidiGetTime" "MidiSendIm" "MidiSend"
            "MidiSendAt" "MidiCountEvs" "MidiGetEv" "MidiAvailEv" 
"MidiFlushEvs" "MidiReadSync" "MidiWriteSync"
            "MidiCall" "MidiTask" "MidiDTask" "MidiForgetTask" 
"MidiCountDTasks" "MidiFlushDTasks" "MidiExec1DTask"
            "MidiNewCell" "MidiFreeCell" "MidiTotalSpace" "MidiGrowSpace" 
"MidiShare"
"midievptr"   
   ))

(in-package :MIDISHARE)

;;;---------------------------------------------------------------------
;;;  C   declarations and constants
;;;---------------------------------------------------------------------

(defvar *CTYPES-DONE* nil)
(when (not *ctypes-done*)
  (defctype -void   :void)
  (defctype -bool   :short-bool)
  (defctype -uint   :unsigned-short)
  (defctype -dword  :unsigned-long)
  (defctype -char   :char)
  (defctype -lpstr  (-char **))
  (defctype -lpcstr -lpstr)
  (defctype -lpcstr -lpstr)
  (defctype Byte :unsigned-char)
  (setf *ctypes-done* t))

;;;---------------------------------------------------------------------
;; Constant definition for every type of MidiShare events
;;;---------------------------------------------------------------------

(defconstant typeNote 0          "a note with pitch, velocity and 
duration")
(defconstant typeKeyOn 1         "a key on with pitch and velocity")
(defconstant typeKeyOff 2        "a key off with pitch and velocity")
(defconstant typeKeyPress 3      "a key pressure with pitch and 
pressure value")
(defconstant typeCtrlChange 4    "a control change with control 
number and control value")
(defconstant typeProgChange 5    "a program change with program 
number")
(defconstant typeChanPress 6     "a channel pressure with pressure 
value")
(defconstant typePitchWheel 7    "a pitch bender with lsb and msb 
of the 14-bit value")
(defconstant typePitchBend 7     "a pitch bender with lsb and msb 
of the 14-bit value")
(defconstant typeSongPos 8       "a song position with lsb and msb of 
the 14-bit position")
(defconstant typeSongSel 9       "a song selection with a song 
number")
(defconstant typeClock 10        "a clock request (no argument)")
(defconstant typeStart 11        "a start request (no argument)")
(defconstant typeContinue 12     "a continue request (no 
argument)")
(defconstant typeStop 13         "a stop request (no argument)")
(defconstant typeTune 14         "a tune request (no argument)")
(defconstant typeActiveSens 15   "an active sensing code (no 
argument)")
(defconstant typeReset 16        "a reset request (no argument)")
(defconstant typeSysEx 17        "a system exclusive with any 
number of data bytes. Leading $F0 and tailing $F7 are automatically 
supplied by MidiShare and MUST NOT be included by the user")
(defconstant typeStream 18       "a special event with any number 
of data and status bytes sended without any processing")
(defconstant typePrivate 19      "a private event for internal use 
with 4 32-bits arguments")
(defconstant typeProcess 128     "an interrupt level task with a 
function adress and 3 32-bits arguments")
(defconstant typeDProcess 129    "a foreground level task with a 
function adress and 3 32-bits arguments")
(defconstant typeQFrame 130      "a quarter frame message with a 
type from 0 to 7 and a value")

(defconstant typeCtrl14b	131)
(defconstant typeNonRegParam	132)
(defconstant typeRegParam	133)

(defconstant typeSeqNum		134)
(defconstant typeText		135)
(defconstant typeCopyright	136)
(defconstant typeSeqName	137)
(defconstant typeInstrName	138)
(defconstant typeLyric		139)
(defconstant typeMarker		140)
(defconstant typeCuePoint	141)
(defconstant typeChanPrefix	142)
(defconstant typeEndTrack	143)
(defconstant typeTempo		144)
(defconstant typeSMPTEOffset	145)

(defconstant typeTimeSign	146)
(defconstant typeKeySign	147)
(defconstant typeSpecific	148)

(defconstant typeReserved 149    "events reserved for futur use")
(defconstant typedead 255        "a dead task. Used by MidiShare to 
forget and inactivate typeProcess and typeDProcess tasks")

;; Constant definition for every MidiShare error code

(defconstant MIDIerrSpace -1	 "too many applications")
(defconstant MIDIerrRefNum -2	 "bad reference number")
(defconstant MIDIerrBadType -3   "bad event type")
(defconstant MIDIerrIndex -4	 "bad index")

;; Constant definition for the Macintosh serial ports

(defconstant ModemPort 0	 "Macintosh modem port")
(defconstant PrinterPort 1	 "Macintosh printer port")

;; Constant definition for the synchronisation modes

(defconstant MidiExternalSync #x8000	 "Bit-15 set for external 
synchronisation")
(defconstant MidiSyncAnyPort #x4000	 "Bit-14 set for 
synchronisation on any port")

;; Constant definition for MidiShare world changes

(defconstant MIDIOpenAppl 1      "an application was opened")
(defconstant MIDICloseAppl 2     "an application was closed")
(defconstant MIDIChgName 3       "an application name was 
changed")
(defconstant MIDIChgConnect 4    "a connection was changed")
(defconstant MIDIOpenModem 5     "Modem port was opened")
(defconstant MIDICloseModem 6    "Modem port was closed")
(defconstant MIDIOpenPrinter 7   "Printer port was opened")
(defconstant MIDIClosePrinter 8  "Printer port was closed")
(defconstant MIDISyncStart 9     "SMPTE synchronisation just start")
(defconstant MIDISyncStop 10     "SMPTE synchronisation just 
stop")

;;;---------------------------------------------------------------------
;;; DLL names
;;;---------------------------------------------------------------------

(defconstant MSHACL-DLL "mshacl.dll")
(defconstant MSHARE-DLL "mshare.dll")
(defconstant *string* (callocate (char 256)))


;;;---------------------------------------------------------------------
;;; from the MSHACL.dll
;;;---------------------------------------------------------------------

(defun-dll SetDate ((ev -dword)(date :unsigned-long))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLSETDATE"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetDate ((ev -dword))
  :return-type  :long
  :library-name mshacl-dll
  :entry-name   "ACLGETDATE"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll SetLink ((ev -dword)(ev1 -dword))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLSETLINK"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetLink ((ev -dword))
  :return-type  -dword
  :library-name mshacl-dll
  :entry-name   "ACLGETLINK"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll SetType ((ev -dword)(type Byte))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLSETTYPE"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetType((ev -dword))
  :return-type  Byte
  :library-name mshacl-dll
  :entry-name   "ACLGETTYPE"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll SetRefnum ((ev -dword)(ref Byte))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLSETREFNUM"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetRefnum((ev -dword))
  :return-type  Byte
  :library-name mshacl-dll
  :entry-name   "ACLGETREFNUM"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll SetChan ((ev -dword)(chan Byte))
  :return-type Byte
  :library-name mshacl-dll
  :entry-name   "ACLSETCHAN"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetChan((ev -dword))
  :return-type  :char
  :library-name mshacl-dll
  :entry-name   "ACLGETCHAN"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll SetPort ((ev -dword)(port Byte))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLSETPORT"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetPort((ev -dword))
  :return-type  Byte
  :library-name mshacl-dll
  :entry-name   "ACLGETPORT"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll SetLinkST ((ev -dword)(ev1 -dword))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLSETLINKST"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetLinkST ((ev -dword))
  :return-type  -dword
  :library-name mshacl-dll
  :entry-name   "ACLGETLINKST"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetName ((refNum :short)(name (:char *)))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLMIDIGETNAME"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll SetFirstEv ((seq -dword) (ev -dword))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLSETFIRSTEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetFirstEv ((seq -dword))
  :return-type  -dword
  :library-name mshacl-dll
  :entry-name   "ACLGETFIRSTEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll SetLastEv ((seq -dword) (ev -dword))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLSETLASTTEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll GetLastEv ((seq -dword))
  :return-type  -dword
  :library-name mshacl-dll
  :entry-name   "ACLGETLASTTEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll AcceptPort ((f -dword) (a Byte) (v :short))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLACCEPTPORT"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll AcceptChan((f -dword) (a Byte) (v :short))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLACCEPTCHAN"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll AcceptType ((f -dword) (a Byte) (v :short))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLACCEPTTYPE"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll NewFilter ()
  :return-type  -dword
  :library-name mshacl-dll
  :entry-name   "ACLNEWFILTER"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll DisposeFilter ((f -dword))
  :return-type  :void
  :library-name mshacl-dll
  :entry-name   "ACLDISPOSEFILTER"
  :16-bit       t
  :call-mode    :pascal)

;;;---------------------------------------------------------------------
;;;  MidiShare  calls
;;;---------------------------------------------------------------------

(defun-dll MidiGetVersion ()
   ;;Give MidiShare version as a fixnum. For example 131 as result"
   ;;means  version 1.31"
  :return-type  :short
  :library-name mshare-dll
  :entry-name   "MIDIGETVERSION"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiCountAppls ()
  ;;Give the number of MidiShare applications currently opened"
  :return-type  :short
  :library-name mshare-dll
  :entry-name   "MIDICOUNTAPPLS"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetIndAppl ((index :short))
  ;;Give the reference number of a MidiShare application from its 
  ;; index, a fixnum between 1 and (MidiCountAppls)"
  :return-type  :short
  :library-name mshare-dll
  :entry-name   "MIDIGETINDAPPL"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetNamedAppl ((name -lpstr ))
  ;;Give the reference number of a MidiShare application from its 
   ;;name"
  :return-type  :short
  :library-name mshare-dll
  :entry-name   "MIDIGETNAMEDAPPL"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiOpen  ((name -lpstr))
  :return-type  :short
  :library-name mshare-dll
  :entry-name   "MIDIOPEN"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiClose ((refnum :short ))
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDICLOSE"
  :16-bit       t
  :call-mode    :pascal)

(defun MidiGetName (ref)
   (getname ref *string*)
   (subseq *string* 0 (strlen *string*)))

(defun-dll MidiSetName ((refNum :short) (name -lpstr))
  ;;Change the name of a MidiShare application"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISETNAME"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetInfo ((refNum :short))
  ;;Give the 32-bits user defined content of the info field of a 
;;MidiShare application Analogous to window's refcon."
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDIGETINFO"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiSetInfo ((refNum :short) (p -dword))
  ;;Set the 32-bits user defined content of the info field of a 
   ;;MidiShare application.  Analogous to window's refcon."
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISETINFO"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetFilter ((refNum :short))
  ;;Give a pointer to the input filter record of a MidiShare 
   ;;application. Give NIL if no filter is installed"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDIGETFILTER"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiSetFilter ((refNum :short) (p -dword))
  ;;Install an input filter. The argument p is a pointer to a filter 
   ;;record."
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISETFILTER"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetRcvAlarm ((refNum :short))
  ;;Get the adress of the receive alarm"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDIGETRCVALARM"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiSetRcvAlarm ((refNum :short) (alarm (:void *)))
  ;;Install a receive alarm"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISETRCVALARM"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetApplAlarm ((refNum :short))
  ;;Get the adress of the context alarm"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDIGETAPPLALARM"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiSetApplAlarm ((refNum :short) (alarm -dword) )
  ;;Install a context alarm"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISETAPPLALARM"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiConnectInt ((src :short) (dt  :short) (state  :short))
  ;;Connect or disconnect two MidiShare applications"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDICONNECT"
  :16-bit       t
  :call-mode    :pascal)

(defun midiconnect (src dt state)
   (midiconnectint src dt (if state 1 0)))

(defun-dll MidiIsConnectedInt ((src :short) (dt  :short))
 ;;Test if two MidiShare applications are connected"
  :return-type  :short
  :library-name mshare-dll
  :entry-name   "MIDIISCONNECTED"
  :16-bit       t
  :call-mode    :pascal)

(defun midiisconnected (src dt )
  ( if (= 1 (MidiIsConnectedInt src dt)) t nil))


(defun-dll MidiGetPortStateInt ((port Byte))
  ;;Give the state : open or closed, of a MidiPort"
  :return-type  :short
  :library-name mshare-dll
  :entry-name   "MIDIGETPORTSTATE"
  :16-bit       t
  :call-mode    :pascal)

(defun midigetportstate (port)
   ( if (= 1 (midigetportstateint port)) t nil))



(defun-dll MidiSetPortStateInt ((port Byte) (state :short))
  ;;Open or close a MidiPort"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISETPORTSTATE"
  :16-bit       t
  :call-mode    :pascal)

(defun midisetportstate (port state)
   (MidiSetPortStateInt port (if state 1 0)))


(defun-dll MidiFreeSpace ()
  ;;Amount of free MidiShare cells"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDIFREESPACE"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiNewEv ((typeNum :short))
 ;;Allocate a new MidiEvent"
  :return-type -dword
  :library-name mshare-dll
  :entry-name   "MIDINEWEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiCopyEv ((ev -dword))
  ;;Duplicate a MidiEvent"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDICOPYEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiFreeEv ((ev -dword))
  ;;Free a MidiEvent"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIFREEEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiSetField ((ev -dword) (field -dword) (val -dword))
  ;;Set a field of a MidiEvent"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISETFIELD"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetField ((ev -dword) (field -dword))
  ;;Get a field of a MidiEvent"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDIGETFIELD"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiAddField ((ev -dword) (val -dword))
  ;;Append a field to a MidiEvent (only for sysex and stream)"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIADDFIELD"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiCountFields ((ev -dword))
  ;;The number of fields of a MidiEvent"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDICOUNTFIELDS"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiNewSeq ()
  ;;Allocate an empty sequence"
  :return-type   -dword
  :library-name mshare-dll
  :entry-name   "MIDINEWSEQ"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiAddSeq ((seq  -dword) (ev -dword))
  ;;Add an event to a sequence"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIADDSEQ"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiFreeSeq ((seq -dword))
  ;;Free a sequence and its content"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIFREESEQ"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiClearSeq ((seq  -dword))
  ;;Free only the content of a sequence. The sequence become empty"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDICLEARSEQ"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiApplySeq ((seq  -dword) (proc  -dword))
  ;;Call a function for every events of a sequence"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIAPPLYSEQ"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetTime ()
  ;;give the current time"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDIGETTIME"
  :16-bit       t
  :call-mode    :pascal)

;----------------------------  Events  sending------------------------

(defun-dll MidiSendIm ((refNum :short) (ev -dword))
  ;;send an event now"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISENDIM"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiSend ((refNum :short) (ev -dword))
  ;;send an event using its own date"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISEND"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiSendAt ((refNum :short) (ev -dword) (date :long))
 ;;send an event at date <date>"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISENDAT"
  :16-bit       t
  :call-mode    :pascal)

;---------------------------- Reception d'vnements ---------------


(defun-dll MidiCountEvs ((refNum :short))
  ;;Give the number of events waiting in the reception fifo"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDICOUNTEVS"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetEv ((refNum :short))
  ;;Read an event from the reception fifo"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDIGETEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiAvailEv ((refNum :short))
  ;;Get a pointer to the first event in the reception fifo without 
   ;;removing it"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDIAVAILEV"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiFlushEvs ((refNum :short))
  ;;Delete all the events waiting in the reception fifo"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIFLUSHEVS"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiReadSync ((adrMem -dword))
  ;;Read and clear a memory address (not-interruptible)"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDIREADSYNC"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiWriteSync ((adrMem -dword) (val -dword))
  ;;write if nil into a memory address (not-interruptible)"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIWRITESYNC"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiCall ((proc -dword) (date  :long)(refNum :short) (arg1 
:long) (arg2 :long) (arg3 :long))
  ;;Call the routine <proc> at date <date> with arguments <arg1> <arg2> 
;;<arg3>"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDICALL"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiTask ((proc (:void *)) (date  :long)(refNum :short) (arg1 
:long) (arg2 :long) (arg3 :long))
  ;;Call the routine <proc> at date <date> with arguments <arg1> <arg2> 
  ;;<arg3>. Return a pointer to the corresponding typeProcess event"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDITASK"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiDTask ((proc (:void *)) (date  :long)(refNum :short) 
(arg1 :long) (arg2 :long) (arg3 :long))
  ;;Call the routine <proc> at date <date> with arguments <arg1> <arg2> 
;;<arg3>.  Return a pointer to the corresponding typeDProcess event"
  :return-type   -dword
  :library-name mshare-dll
  :entry-name   "MIDIDTASK"
  :16-bit       t
  :call-mode    :pascal)


;; A REVOIR
#|
(defun-dll MidiForgetTaskHdl (thdl)
  "Forget a previously scheduled typeProcess or typeDProcess event 
  created by MidiTask or MidiDTask"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIFORGETTASK"
  :16-bit       t
  :call-mode    :pascal)


;(defun-dll MidiForgetTask (ev)
;"Forget a previously scheduled typeProcess or typeDProcess event 
created by MidiTask or MidiDTask"
;  `(without-interrupts (rset *taskHdl* :ttaskHdl.task ,ev) 
(midiforgetTaskHdl *taskHdl*)))

(defun-dll MidiForgetTask (ev)
  "Forget a previously scheduled typeProcess or typeDProcess event 
created by MidiTask or MidiDTask"
  `(without-interrupts 
    (%stack-block ((taskptr 4))
      (%setf-macptr taskptr ,ev) (midiforgetTaskHdl taskptr))))

|#


(defun-dll MidiCountDTasks ((refNum :short))
  ;;Give the number of typeDProcess events waiting"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDICOUNTDTASKS"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiFlushDTasks ((refNum :short))
  ;;Remove all the typeDProcess events waiting"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIFLUSHDTASKS"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiExec1DTask ((refNum :short))
  ;;Call the next typeDProcess waiting"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIEXEC1DTASK"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiNewCell ()
  ;;Allocate a basic Cell"
  :return-type  -dword
  :library-name mshare-dll
  :entry-name   "MIDINEWCELL"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiFreeCell ((cell -dword))
  ;;Delete a basic Cell"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIFREECELL"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiTotalSpace ()
  ;;Total amount of Cells"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDITOTALSPACE"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGrowSpace ((n :long))
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDIGROWSPACE"
  :16-bit       t
  :call-mode    :pascal)
  
  
; new synchronisation functions

(defun-dll MidiGetSyncInfo ((syncInfo -dword))
  ;;Fill syncInfo with current synchronisation informations"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDIGETSYNCINFO"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiSetSyncMode ((mode :unsigned-short))
  ;;set the MidiShare synchroniation mode"
  :return-type  :void
  :library-name mshare-dll
  :entry-name   "MIDISETSYNCMODE"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiGetExtTime ()
 ;;give the current external time"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDIGETEXTTIME"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiInt2ExtTime ((time :long))
  ;;convert internal time to external time"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDIINT2EXTTIME"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiExt2IntTime ((time :long))
  ;;convert internal time to external time"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDIEXT2INTTIME"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiTime2Smpte ((time :long) (format :short) 
(smpteLocation -dword))
;;convert time to Smpte location"
:return-type  :void
:library-name mshare-dll
:entry-name   "MIDISETAPPLALARM"
:16-bit       t
:call-mode    :pascal)

(defun-dll MidiSmpte2Time ((smpteLocation -dword))
  ;;convert time to Smpte location"
  :return-type  :long
  :library-name mshare-dll
  :entry-name   "MIDISETAPPLALARM"
  :16-bit       t
  :call-mode    :pascal)

(defun-dll MidiShareInt ()
  :return-type  :unsigned-char
  :library-name mshare-dll
  :entry-name   "MIDISHARE"
  :16-bit       t
  :call-mode    :pascal)

(defun midishare ()
   (if (midishareint) t nil))

(provide 'MIDISHARE)

