;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  MSHEXT.lisp
;;  Copyright (c) 1995, GRAME.  All rights reserved.
;;
;;  This file contains definitions for records and Pascal style routines, 
;;  used for interfacing ACL with MidiShare 1.52, real-time multitasking 
;;  Midi operating system.
;;
;;  Description:  MidiShare extension to Allegro Common Lisp
;;  Date:        30-11-95
;;; --------------------------------------------------------------------
 

(in-package :midishare)

(export '(midi-check-ptr midi-new-cell midi-free-cell 
          midi-new-ev midi-copy-ev midi-free-ev 
          
          midi-list-fields 
          midi-string-date midi-date-string midi-string-type midi-string-ev 
          
          note key-on key-off key-press ctrl-change 
          prog-change chan-press pitch-bend song-pos song-sel 
          clock start stop cont tune active-sens reset 
          sys-ex stream qframe ctrl14b non-reg-param reg-param seq-num text-event
          copyright seq-name instr-name lyric marker cue-Point specific 
          chan-prefix end-track tempo-change smpte-offset time-sign key-sign private 
          
          midi-new-filter midi-free-filter midi-modify-filter 
          
          firstEv lastEv link linkSE linkST 

          date type ref port chan pitch vel dur kpress ctrl param val pgm bend clk song 
          fields fmsg fcount info field fieldsList num text prefix tempo seconds subframes
          tsnum tsdenom tsclick tsquarter alteration minor-scale
          
          FilterBit AcceptPort AcceptType AcceptChan))

;;           Macros to acces fields of Midishare events
;;==============================================================

;;...................................................................:for sequence 
(defmacro firstEv (s &optional (e nil e?))
"read or set the first event of a sequence"
  (if e? `(SetFirstEv ,s ,e)  `(GetFirstEv ,s )))

(defmacro lastEv (s &optional (e nil e?))
"read or set the last event of a sequence"
  (if e?  `(SetLastEv ,s ,e)  `(GetLastEv ,s )))
;;...................................................................: common fields
(defmacro link (e &optional (d nil d?))
"read or set the link of an event"
  (if d?  `(SetLink ,e ,d) `(GetLink ,e )))

(defmacro linkSE (e &optional (d nil d?))
"read or set the link of an SEXevent "
   (if d?  `(SetLinkST ,e ,d) `(GetLinkST ,e )))

(defmacro linkST (e &optional (d nil d?))
 "read or set the link of an STevent "
 (if d? `(SetLinkST ,e ,d)`(GetLinkST ,e )))

(defmacro date (e &optional (d nil d?))
"read or set the date of an event"
    (if d? `(SetDate ,e ,d)`(GetDate ,e )))

(defmacro type (e &optional(v nil v?))
"read or set the type of an event. Be careful in 
 modifying the type of an event"
   (if v? `(SetType ,e ,v) `(GetType ,e )))

(defmacro ref (e &optional (v nil v?))
"read or set the reference number of an event"
   (if v? `(SetRefnum,e ,v)`(GetRefnum ,e )))

(defmacro port (e &optional(v nil v?))
"read or set the port number of an event"
  (if v?  `(SetPort,e ,v) `(GetPort ,e )))

(defmacro chan (e &optional(v nil v?))
"read or set the chan number of an event"
  (if v?  `(SetChan,e ,v)  `(GetChan ,e )))
;;...................................................................: for notes 
(defmacro pitch (e &optional v)
"read or set the pitch of an event"
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))

(defmacro vel (e &optional v)
"read or set the velocity of an event"
   (if v `(midisetfield ,e 1 ,v)`(midigetfield ,e 1)))

(defmacro dur (e &optional v)
"read or set the duration of an event"
 (if v `(midisetfield ,e 2 ,v) `(midigetfield ,e 2)))
;;...................................................................: for key-press
(defmacro kpress (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))
;;..................................................................: for ctrl-change
(defmacro ctrl (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))
;;..................................................................: for parameters
(defmacro param (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))
;;...................................................................:sequence number
(defmacro num (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))
;;..................................................................:chan prefix
(defmacro prefix (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))
;;...................................................................: tempo
(defmacro tempo (e &optional v)
  (if v `(midisetfield ,e 0 ,v)`(midigetfield ,e 0)))
;;...................................................................: seconds (smpte-offset)
(defmacro seconds (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))
;;...................................................................: subframes (smpte-offset)
(defmacro subframes (e &optional v)
  (if v `(midisetfield ,e 1 ,v) `(midigetfield ,e 1)))
;;...................................................................: ctral-change value
(defmacro val (e &optional v)
  (if v `(midisetfield ,e 1 ,v) `(midigetfield ,e 1)))
;;...................................................................: prog-change value
(defmacro pgm (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))
;;...................................................................: pitch-bend value
(defmacro bend (e &optional v)
  "read or set the bend value of an event"
  (if v
    `(multiple-value-bind (ms7b ls7b) (floor (+ ,v 8192) 128)
       (midisetfield ,e 0  ls7b) 
       (midisetfield ,e 1  ms7b))
    `(- (+ (midigetfield ,e 0) (* 128 (midigetfield ,e 1))) 8192)))
;;...................................................................: for song-pos
(defmacro clk (e &optional v)
  (if v
    `(multiple-value-bind (ms7b ls7b) (floor (round (/ ,v 6)) 128)
       (midisetfield ,e 0  ls7b) 
       (midisetfield ,e 1  ms7b))
    `(* 6 (+ (midigetfield ,e 0) (* 128 (midigetfield ,e 1)))) ))
;;...................................................................: for song-sel
(defmacro song (e &optional v)
   (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))
;;...................................................................: for sys-ex
(defmacro fields (e &optional v)
  (if v
    `(let ((e ,e)) (mapc #'(lambda (f) (midiaddfield e f)) ,v))
    `(let (l (e ,e))  (dotimes (i (midicountfields e)) (push (midigetfield e i) l)) (nreverse l)) ))
;;...................................................................: for text events
(defmacro text (e &optional s)
  (if s
    `(fields ,e (map 'list #'char-code ,s))
    `(map 'string #'character (fields ,e)) ))
;;...................................................................: for qframe
(defmacro fmsg (e &optional v)
    (if v  `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))

(defmacro fcount (e &optional v)
    (if v `(midisetfield ,e 1 ,v) `(midigetfield ,e 1)))
;;...................................................................: for time-sign
(defmacro tsnum (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))

(defmacro tsdenom (e &optional v)
  (if v `(midisetfield ,e 1 ,v) `(midigetfield ,e 1)))

(defmacro tsclick (e &optional v)
  (if v `(midisetfield ,e 2 ,v) `(midigetfield ,e 2)))

(defmacro tsquarter (e &optional v)
  (if v `(midisetfield ,e 3 ,v) `(midigetfield ,e 3)))
;;...................................................................: for key-sign
(defmacro alteration (e &optional v)
  (if v `(midisetfield ,e 0 ,v) `(midigetfield ,e 0)))

(defmacro minor-scale (e &optional v)
  (if v `(midisetfield ,e 1 (if ,v 1 0))`(= 1 (midigetfield ,e 1))))

;;..................................................................: diverses
;;(defmacro info (e &optional d)
;;"read or set the info of an event"
;;  (if e?
;;     `(SetInfo ,e ,d)
 ;;   `(GetInfo ,e )))

(defmacro field (e &optional f v)
"give the number of fields or read or set a particular field of an event"
  (if f (if v `(midiSetField ,e ,f ,v) `(midiGetField ,e ,f)) `(midiCountFields ,e)))

(defun fieldsList (e &optional (n 4))
"collect all the fields of an event into a list"
  (let (l)
    (dotimes (i (min n (midicountfields e)))
      (push (midigetfield e i) l))
    (nreverse l)))

(defun midi-list-fields (e &optional (n 4))
   (if (or (and (>= (type e) typetext) (<= (type e) typecuepoint))
                (= (type e ) typespecific))
           (text e)
      (let (l)
           (dotimes (i (min n (midicountfields e)))
              (push (midigetfield e i) l))
           (nreverse l))))

;;          Events printing
;;==============================================================
;;...................................................................: midi-string-date
(defun midi-string-date (d)
  (let (hh mn ss ms)
    (multiple-value-setq (ss ms) (floor d 1000))
    (multiple-value-setq (mn ss) (floor ss 60))
    (multiple-value-setq (hh mn) (floor mn 60))
    (format nil "~2,'0d:~2,'0d:~2,'0d.~3,'0d" hh mn ss ms)))
;;..................................................................: midi-date-string
(defun midi-date-string (s)
  (let ((d 0) (v 0)(n 0))
    (dotimes (i (length s))
      (cond ((setq n (digit-char-p (char s i)))
             (setq v (+ (* v 10) n)))
            ((eq (char s i) #\:)
             (setq d (* (+ d v) 60))
             (setq v 0))
            ((eq (char s i) #\.)
             (setq d (* (+ d v) 1000))
             (setq v 0))))
    (+ d v)))
;;...................................................................: midi-string-type
(defun midi-string-type (n)
  "convert a MidiShare event type to its name, a string."
  (let* ((lName '((0 . "Note") (1 . "KeyOn") (2 . "KeyOff") (3 . "KeyPress")
                  (4 . "CtrlChange") (5 . "ProgChange") (6 . "ChanPress") (7 . "PitchBend")
                  (8 . "SongPos") (9 . "SongSel") (10 . "Clock") (11 . "Start")
                  (12 . "Cont") (13 . "Stop") (14 . "Tune") (15 . "ActiveSens")
                  (16 . "Reset") (17 . "SysEx") (18 . "Stream") 
                  (128 . "Process") (129 . "DProcess") (130 . "QFrame")
                  (131 . "Ctrl14b") (132 . "NonRegParam") (133 . "RegParam")
                  (134 . "SeqNum") (135 . "Text:") (136 . "Copyright:")
                  (137 . "SeqName:") (138 . "InstrName:") (139 . "Lyric:")
                  (140 . "Marker:") (141 . "CuePoint:") (142 . "ChanPrefix")
                  (143 . "EndTrack") (144 . "Tempo") (145 . "SMPTEOffset")
                  (146 . "TimeSign") (147 . "KeySign") (148 . "Specific:")
                  (255 . "Dead")))
         (l (assoc n lName)) )
    (cond (l 
           (cdr l))
          ((< n typeProcess)
           "Private")
          (t
           "Undefined") )))
;;...................................................................: midi-string-ev
(defun midi-string-ev (e &optional (f 4))
  (unless (= 0 e)
    (list 
    (format nil "~A ~3D/~4A ~A ~:a" 
                (midi-string-date (date e)) 
                (port e) (chan e) 
                (midi-string-type (type e)) 
                (midi-list-fields e f)))))

;;       Creation of the different  MidiShare events types
;;==============================================================

(defmacro midi-check-ptr (e)
  `(if (= 0 e) (error "MidiShare run out of memory")))
;;...................................................................: midi-new-cell
(defun midi-new-cell ()
  (let ((e (midinewcell)))
    (midi-check-ptr e) e))
;;...................................................................: midi-free-cell
(defun midi-free-cell (e)
  (midifreecell e))
;;...................................................................: midi-new-ev
(defun midi-new-ev (evtype )
  (let ((e (midinewev evtype)))
    (midi-check-ptr e)   e))
;;...................................................................: midi-copy-ev
(defun midi-copy-ev (e1)
  (let ((e2 (midicopyev e1)))
    (midi-check-ptr e2)  e2))
;;...................................................................: midi-free-ev
(defun midi-free-ev (e)
  (midifreeev e))
;;...................................................................: note

(defun note (&key (date 0)(pitch 60)(vel 80)(dur 125)(chan 0)(port 0) &aux e)
  "create a note event"
  (setq e (midi-new-ev typeNote))
  (date e (if (stringp date) (midi-date-string date) date))
  (pitch e pitch)
  (vel e vel)
  (dur e dur)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: key-on
(defun key-on (&key (date 0)(pitch 60)(vel 80)(chan 0)(port 0) &aux e)
  "create a key on event"
  (setq e (midi-new-ev typeKeyOn))
  (date e (if (stringp date) (midi-date-string date) date))
  (pitch e pitch)
  (vel e vel)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: key-off
(defun key-off (&key (date 0)(pitch 60)(vel 80)(chan 0)(port 0) &aux e)
  "create a key off event"
  (setq e (midi-new-ev typeKeyOff))
  (date e (if (stringp date) (midi-date-string date) date))
  (pitch e pitch)
  (vel e vel)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: key-press
(defun key-press (&key (date 0)(pitch 60)(kpress 80)(chan 0)(port 0) &aux e)
  "create a key pressure event"
  (setq e (midi-new-ev typekeypress))
  (date e (if (stringp date) (midi-date-string date) date))
  (pitch e pitch)
  (kpress e kpress)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: ctrl-change
(defun ctrl-change (&key (date 0)(ctrl 7)(val 127)(chan 0)(port 0) &aux e)
  "create a control change event"
  (setq e (midi-new-ev typeCtrlChange))
  (date e (if (stringp date) (midi-date-string date) date))
  (ctrl e ctrl)
  (val e val)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: ctrl14b
(defun ctrl14b (&key (date 0)(ctrl 7)(val 127)(chan 0)(port 0) &aux e)
  "create a control change event"
  (setq e (midi-new-ev typeCtrl14b))
  (date e (if (stringp date) (midi-date-string date) date))
  (ctrl e ctrl)
  (val e val)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: non registred parameter
(defun non-reg-param (&key (date 0)(param 0)(val 0)(chan 0)(port 0) &aux e)
  "create a non registred parameter event"
  (setq e (midi-new-ev typeNonRegParam))
  (date e (if (stringp date) (midi-date-string date) date))
  (param e param)
  (val e val)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: registred parameter
(defun reg-param (&key (date 0)(param 0)(val 0)(chan 0)(port 0) &aux e)
  "create a registred parameter event"
  (setq e (midi-new-ev typeRegParam))
  (date e (if (stringp date) (midi-date-string date) date))
  (param e param)
  (val e val)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: sequence number
(defun seq-num (&key (date 0)(num 0)(chan 0)(port 0) &aux e)
  "create a sequence number event"
  (setq e (midi-new-ev typeSeqNum))
  (date e (if (stringp date) (midi-date-string date) date))
  (num e num)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: text event
(defun text-event (&key (date 0)(text "untitled")(chan 0)(port 0) &aux e)
  "create a text event"
  (setq e (midi-new-ev typeText))
  (date e (if (stringp  date) (midi-date-string date) date))
  (text e text)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: copyright event
(defun copyright (&key (date 0)(text "untitled")(chan 0)(port 0) &aux e)
  "create a copyright event"
  (setq e (midi-new-ev typeCopyright))
  (date e (if (stringp date) (midi-date-string date) date))
  (text e text)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: seqname event
(defun seq-name (&key (date 0)(text "untitled")(chan 0)(port 0) &aux e)
  "create a seqname event"
  (setq e (midi-new-ev typeseqname))
  (date e (if (stringp date) (midi-date-string date) date))
  (text e text)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: instrname event
(defun instr-name (&key (date 0)(text "untitled")(chan 0)(port 0) &aux e)
  "create a instrname event"
  (setq e (midi-new-ev typeinstrname))
  (date e (if (stringp date) (midi-date-string date) date))
  (text e text)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: lyric event
(defun lyric (&key (date 0)(text "untitled")(chan 0)(port 0) &aux e)
  "create a lyric event"
  (setq e (midi-new-ev typelyric))
  (date e (if (stringp date) (midi-date-string date) date))
  (text e text)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: marker event
(defun marker (&key (date 0)(text "untitled")(chan 0)(port 0) &aux e)
  "create a marker event"
  (setq e (midi-new-ev typemarker))
  (date e (if (stringp date) (midi-date-string date) date))
  (text e text)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: cue Point event
(defun cue-point (&key (date 0)(text "untitled")(chan 0)(port 0) &aux e)
  "create a cue Point event"
  (setq e (midi-new-ev typeCuePoint))
  (date e (if (stringp date) (midi-date-string date) date))
  (text e text)
  (chan e chan)
  (port e port)
  e)
;;..................................................................: specific event
(defun specific (&key (date 0)(fields ())(chan 0)(port 0) &aux e)
  "create a specific event"
  (setq e (midi-new-ev typeSpecific))
  (date e (if (stringp date) (midi-date-string date) date))
  (fields e fields)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: channel prefix
(defun chan-prefix (&key (date 0)(prefix 0)(chan 0)(port 0) &aux e)
  "create a channel prefix event"
  (setq e (midi-new-ev typeChanPrefix))
  (date e (if (stringp date) (midi-date-string date) date))
  (prefix e prefix)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: end of track
(defun end-track (&key (date 0)(chan 0)(port 0) &aux e)
  "create an end of track event"
  (setq e (midi-new-ev typeEndTrack))
  (date e (if (stringp date) (midi-date-string date) date))
  (chan e chan)
  (port e port)
  e)
;;...................................................................: tempo-change
(defun tempo-change (&key (date 0)(tempo 0)(chan 0)(port 0) &aux e)
  "create a tempo change event"
  (setq e (midi-new-ev typeTempo))
  (date e (if (stringp date) (midi-date-string date) date))
  (tempo e tempo)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: smpte-offset
(defun smpte-offset (&key (date 0) (seconds 0) (subframes 0) (chan 0)(port 0) &aux e)
  "create an smpte offset event"
  (setq e (midi-new-ev typeSmpteOffset))
  (date e (if (stringp date) (midi-date-string date) date))
  (seconds e seconds)
  (subframes e subframes)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: time-signature
(defun time-Sign (&key (date 0) (tsnum 0) (tsdenom 0) (tsclick 0)(tsquarter 0)(port 0) &aux 
e)
  "create a time signature event"
  (setq e (midi-new-ev typeTimeSign))
  (date e (if (stringp date) (midi-date-string date) date))
  (tsnum e tsnum)
  (tsdenom e tsdenom)
  (tsclick e tsclick)
  (tsquarter e tsquarter)
  (port e port)
  e)
;;...................................................................: key-signature
(defun key-Sign (&key (date 0) (alteration 0) (minor-scale nil) (port 0) &aux e)
  "create a key signature event"
  (setq e (midi-new-ev typeKeySign))
  (date e (if (stringp date) (midi-date-string date) date))
  (alteration e alteration)
  (minor-scale e minor-scale)
  (port e port)
  e)
;;...................................................................: prog-change
(defun prog-change (&key (date 0)(pgm 0)(chan 0)(port 0) &aux e)
  "create a program change event"
  (setq e (midi-new-ev typeProgChange))
  (date e (if (stringp date) (midi-date-string date) date))
  (pgm e pgm)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: chan-press
(defun chan-press (&key (date 0)(press 80)(chan 0)(port 0) &aux e)
"create a channel pressure event"
  (setq e (midi-new-ev typeChanPress))
  (date e (if (stringp date) (midi-date-string date) date))
  (pitch e press)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: pitch-bend
(defun pitch-bend (&key (date 0)(bend 0)(chan 0)(port 0) &aux e)
  "create a pitch bender event. The argument bend a value between -8192..+8191"
  (setq e (midi-new-ev typePitchWheel))
  (date e (if (stringp date) (midi-date-string date) date))
  (bend e bend)
  (chan e chan)
  (port e port)
  e)
;;...................................................................: song-pos
(defun song-pos (&key (date 0)(clk 0)(port 0) &aux e)
  "create a song position pointer event. ClkNum is IN MIDI CLOCK and divided 
automaticaly by 6"
  (setq e (midi-new-ev typeSongPos))
  (date e (if (stringp date) (midi-date-string date) date))
  (clk e clk)
  (port e port)
  e)
;;...................................................................: song-sel
(defun song-sel (&key (date 0)(song 0)(port 0) &aux e)
  "create a song select event"
  (setq e (midi-new-ev typeSongSel))
  (date e (if (stringp date) (midi-date-string date) date))
  (song e song)
  (port e port)
  e)
;;...................................................................: clock
(defun clock (&key (date 0)(port 0) &aux e)
  "create a clock event"
  (setq e (midi-new-ev typeClock))
  (date e (if (stringp date) (midi-date-string date) date))
  (port e port)
  e)
;;...................................................................: start
(defun start (&key (date 0)(port 0) &aux e)
  "create a start event"
  (setq e (midi-new-ev typeStart))
  (date e (if (stringp date) (midi-date-string date) date))
  (port e port)
  e)
;;...................................................................: stop
(defun stop (&key (date 0)(port 0) &aux e)
  "create a stop event"
  (setq e (midi-new-ev typeStop))
  (date e (if (stringp date) (midi-date-string date) date))
  (port e port)
  e)
;;...................................................................: cont
(defun cont (&key (date 0)(port 0) &aux e)
  "create a continue event"
  (setq e (midi-new-ev typeContinue))
  (date e (if (stringp date) (midi-date-string date) date))
  (port e port)
  e)
;;...................................................................: tune
(defun tune (&key (date 0)(port 0) &aux e)
  "create a tune event"
  (setq e (midi-new-ev typeTune))
  (date e (if (stringp date) (midi-date-string date) date))
  (port e port)
  e)
;;...................................................................: active-sens
(defun active-sens (&key (date 0)(port 0) &aux e)
  "create a active sensing event"
  (setq e (midi-new-ev typeActiveSens))
  (date e (if (stringp date) (midi-date-string date) date))
  (port e port)
  e)
;;...................................................................: reset
(defun reset (&key (date 0)(port 0) &aux e)
  "create a reset event"
  (setq e (midi-new-ev typeReset))
  (date e (if (stringp date) (midi-date-string date) date))
  (port e port)
  e)
;;...................................................................: sys-ex
(defun sys-ex (&key (date 0)(fields ())(chan 0)(port 0) &aux e)
  "create a system exclusive event from a list of values. 
 Leading F0 and tailing F7 MUST NOT be included in the list. 
 They are provided by MidiShare when the event is actually sended to output"
  (setq e (midi-new-ev typeSysEx))
  (date e (if (stringp date) (midi-date-string date) date))
  (chan e chan)
  (port e port)
  (fields e fields)
  e)
;;...................................................................: stream
(defun stream-ev (&key (date 0)(fields ())(port 0) &aux e)
"create an arbitrary stream of bytes (both status and data) 
 sended by MidiShare without any processing."
  (setq e (midi-new-ev typeStream))
  (date e (if (stringp date) (midi-date-string date) date))
  (port e port)
  (fields e fields)
  e)
;;...................................................................: qframe
(defun qframe (&key (date 0)(fmsg 0) (fcount 0)(port 0) &aux e)
"create a quarter frame event"
  (setq e (midi-new-ev typeQFrame))
  (date e (if (stringp date) (midi-date-string date) date))
  (fmsg e fmsg)
  (fcount e fcount)
  (port e port)
  e)
;;...................................................................: private
(defun private (&key (date 0) (subtype 0) 
                     (field0 0) (field1 0) 
                     (field2 0) (field3 0) &aux e)
  "create a private event. subtype, an integer between 0..108 is a subtype for the private 
event."
  (setq e (midi-new-ev (+ typePrivate subtype)))
  (date e (if (stringp date) (midi-date-string date) date))
  (Field e 0 field0)
  (Field e 1 field1)
  (Field e 2 field2)
  (Field e 3 field3)
  e)

;;             Filters  creation and modification
;;==============================================================
;;...................................................................: midi-new-filter
(defun midi-new-filter (&key chan port type)
  (let ((f (newfilter)))
    (cond ((eq chan t) (dotimes (i 16) (acceptchan f i 1)))
          ((numberp chan) (acceptchan f chan 1))
          (t (dolist (i chan) (acceptchan f i 1))))
    (cond ((eq type t) (dotimes (i 256) (accepttype f i 1)))
          ((numberp type) (accepttype f type 1))
          (t (dolist (i type) (accepttype f i 1))))
    (cond ((eq port t) (dotimes (i 256) (acceptport f i 1)))
          ((numberp port) (acceptport f port 1))
          (t (dolist (i port) (acceptport f i 1)))) 
    f))
;;...................................................................: midi-free-filter
(defun midi-free-filter (f)
  (unless (= 0 f)
    (disposefilter f)))

;;...................................................................: midi-modify-filter
(defun midi-modify-filter (f &key accept chan port type)
  (unless (= 0 f)
    (setq accept (if accept 1 0))
    (cond ((eq chan t) (dotimes (i 16) (acceptchan f i accept)))
          ((numberp chan) (acceptchan f chan accept))
          (t (dolist (i chan) (acceptchan f i accept))))
    (cond ((eq type t) (dotimes (i 256) (accepttype f i accept)))
          ((numberp type) (accepttype f type accept))
          (t (dolist (i type) (accepttype f i accept))))
    (cond ((eq port t) (dotimes (i 256) (acceptport f i accept)))
          ((numberp port) (acceptport f port accept))
          (t (dolist (i port) (acceptport f i accept))))))

