/* ==================================================================

msCTRL.C

(c) GRAME 1995

================================================================== */

#ifndef __MidiShare__
#include "mshare.h"
#endif

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __msCtrl__
#include "msctrl.h"
#endif


// ------------------------------------------------------------------
// CONSTANTS - DATA TYPES
// ------------------------------------------------------------------
typedef void (*ScrollActionProcPtr) (void);
typedef char State[8], *StatePtr;
enum {kPort, kChan, kProgChge, kVol, kModulation, kPan, kPitch, kVel};

#define kDefaultXPos 100
#define kDefaultYPos 100

#define kNoteLen		200
#define kPageStep		10

#define WriteState WritePrivateProfileString
#define GetState	 GetPrivateProfileInt

// ------------------------------------------------------------------
// FUNCTIONS PROTOTYPES
// ------------------------------------------------------------------
void AlertUser(LPCSTR);
Boolean SetUpMidi(void);
void SendNote(void);
void SendProgChge(void);
void SendVol(void);
void SendMod(void);
void SendPan(void);

void FAR PASCAL _export ReceiveEvents(register short);

void LoadState (StatePtr state);
void SaveState (StatePtr state);
BOOL FAR PASCAL _export AboutDlgProc(HWND hDlg,UINT message,
													UINT wParam,LONG lParam);


// ------------------------------------------------------------------
// VARIABLES
// ------------------------------------------------------------------
HANDLE	ghInst;
HWND		ghWnd;
HWND		abouthDlg;
DLGPROC  AboutDlgProcInst;

State gState;
ScrollActionProcPtr	gScrollAction[]= {
		  SendNote,
		  SendNote,
		  SendProgChge,
		  SendVol,
		  SendMod,
		  SendPan,
		  SendNote,
		  SendNote
};

// MidiShare variables
short myRefNum;		// numero de l'application
MidiName AppliName = "UniversalCtrl";

// for private profile file
char * profileName = "msctrl.ini";
char * sectionName = "Window position";
char * xEntryName  = "xpos";
char * yEntryName  = "ypos";
char * StateSectionName = "Application state";
char * portEntryName			= "port";
char * chanEntryName			= "chan";
char * progChgeEntryName	= "progChge";
char * volEntryName			= "vol";
char * modulationEntryName = "modulation";
char * panEntryName			= "pan";
char * pitchEntryName		= "pitch";
char * velEntryName			= "vel";
int	dlgXPos,dlgYPos;


/* ------------------------------------------------------------------

APPLICATION MAIN ENTRY POINT

------------------------------------------------------------------ */
int PASCAL WinMain(HINSTANCE hInstance,
			HINSTANCE hPrevInstance,
			LPSTR lpszCmdLine,
			int nCmdShow)
{
  if ( hPrevInstance ) return FALSE ;
  if ( !SetUpMidi() ) return 0;
  ghInst = hInstance;
  DialogBox(hInstance, "UniversalCtrl", NULL,
		 (DLGPROC)MakeProcInstance(UniversalCtrlDlgProc,hInstance));
  MidiClose(myRefNum);
  return TRUE;
}

/* ------------------------------------------------------------------
					Initializations
------------------------------------------------------------------ */
Boolean InitScroller(HWND hDlg, int ID, int txtID, int min, int max, int state)
{
	HWND		hCtrl;

	hCtrl = GetDlgItem (hDlg, ID);
	if (hCtrl) {
		SetScrollRange(hCtrl, SB_CTL, min, max, FALSE);
		SetScrollPos  (hCtrl, SB_CTL, state, FALSE);
		SetDlgItemInt (hDlg,  txtID,  txtID==IDD_TX_CHAN ? state+1 : state, FALSE);
		return true;
	}
	return false;
}

//______________________________________________________________________
Boolean Initialize(HWND hDlg)
{
	HWND		hCtrl;
	Boolean 	retCode;

	LoadState (gState);
	AboutDlgProcInst= (DLGPROC)MakeProcInstance(AboutDlgProc,ghInst);
	if (!AboutDlgProcInst)
		return false;
	if (!InitScroller (hDlg,IDD_SB_PORT, IDD_TX_PORT, 0, 127, gState[kPort]))
		return false;
	if (!InitScroller (hDlg,IDD_SB_CHAN, IDD_TX_CHAN, 0, 15, gState[kChan]))
		return false;
	if (!InitScroller (hDlg,IDD_SB_PROG, IDD_TX_PROG, 0, 127, gState[kProgChge]))
		return false;
	if (!InitScroller (hDlg,IDD_SB_VOL, IDD_TX_VOL, 0, 127, gState[kVol]))
		return false;
	if (!InitScroller (hDlg,IDD_SB_MOD, IDD_TX_MOD, 0, 127, gState[kModulation]))
		return false;
	if (!InitScroller (hDlg,IDD_SB_PAN, IDD_TX_PAN, 0, 127, gState[kPan]))
		return false;
	if (!InitScroller (hDlg,IDD_SB_PITCH, IDD_TX_PITCH, 0, 127, gState[kPitch]))
		return false;
	if (!InitScroller (hDlg,IDD_SB_VEL, IDD_TX_VEL, 0, 127, gState[kVel]))
		return false;
	SetWindowPos(hDlg, HWND_TOP, dlgXPos, dlgYPos,0,0,SWP_NOSIZE);
	return retCode ;
}


/* ------------------------------------------------------------------
					Scroll bar management
------------------------------------------------------------------ */
void VScroll (HWND hCtrl, WORD part, WORD lp )
{
	short ctrlID, max, i, val;

	ctrlID = GetWindowWord (hCtrl,GWW_ID);
	i 		 = ctrlID - IDD_SB_PORT;
	max	 = ctrlID==IDD_SB_CHAN ? 15 : 127;
	switch (part) {
		case SB_PAGEUP :
			val = min(max, gState[i] + kPageStep);
			break;
		case SB_PAGEDOWN :
			val = max(0, gState[i] - kPageStep);
			break;
		case SB_LINEUP :
			val = min(max, gState[i] + 1);
			break;
		case SB_LINEDOWN :
			val = max(0, gState[i] - 1);
			break;
		case SB_TOP:
			val = max;
			break;
		case SB_BOTTOM :
			val = 0;
			break;
		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
			val = max - lp;
			break;
		default:
			return;
	}
	SetScrollPos (hCtrl, SB_CTL, max - val, TRUE);
	SetDlgItemInt(ghWnd, IDD_TX_PORT+i, i==kChan ? val+1 : val, FALSE);
   gState[i] = val;
	gScrollAction[i]();
}

/* ------------------------------------------------------------------

					main window management

------------------------------------------------------------------ */
BOOL FAR PASCAL _export UniversalCtrlDlgProc(HWND hDlg, UINT message,
						  UINT wParam, LONG lParam)
{
	HWND	hCtrl;
	short ctrlID, max, i;

	switch (message) {
		case WM_INITDIALOG:
			ghWnd= hDlg;
			return Initialize (hDlg);

		case WM_VSCROLL :
			hCtrl = HIWORD(lParam);
			VScroll (hCtrl, wParam, LOWORD(lParam));
			break;

	 case WM_MOVE:
			dlgXPos= LOWORD(lParam) - 5;
			dlgYPos= HIWORD(lParam) - 24;
			break;

	 case WM_COMMAND:
			switch ( wParam ) {
				case AboutButton :
					EnableWindow(GetDlgItem(ghWnd,AboutButton),false);
					DialogBox(ghInst, "AboutDlg", hDlg, AboutDlgProcInst);
					break;
				case IDD_QUITTER :
					if (abouthDlg) EndDialog (abouthDlg, TRUE);
					SaveState(gState);
					EndDialog(hDlg,TRUE);
					break;
			}
			break;
	}
	return FALSE ;
}


/* ------------------------------------------------------------------

 Messages corresponding to user actions

------------------------------------------------------------------ */
void SendNote(void)
{
	 MidiEvPtr e;

	 e = MidiNewEv(typeNote);
	 if (e) {
		Port(e)	= gState[kPort];
		Chan(e)	= gState[kChan];
		Pitch(e) = gState[kPitch];
		Vel(e)	= gState[kVel];
		Dur(e)	= kNoteLen;
		MidiSendIm (myRefNum,e);
	 }
}

void SendCtrl(char ctrlNum, char ctrlVal)
{
	 MidiEvPtr e;

	 e = MidiNewEv(typeCtrlChange);
	 if (e) {
		Port(e)	= gState[kPort];
		Chan(e)	= gState[kChan];
		Data(e)[0]	= ctrlNum;
		Data(e)[1]	= ctrlVal;
		MidiSendIm (myRefNum,e);
	 }
}

void SendVol(void)	{ SendCtrl (7,  gState[kVol]);}
void SendPan(void)	{ SendCtrl (10, gState[kPan]);}
void SendMod(void)	{ SendCtrl (1,  gState[kModulation]);}

void SendProgChge(void)
{
	 MidiEvPtr e;

	 e = MidiNewEv(typeProgChange);
	 if (e) {
		Port(e)	= gState[kPort];
		Chan(e)	= gState[kChan];
		Data(e)[0] = gState[kProgChge];
		MidiSendIm (myRefNum,e);
	 }
}

/* ----------------------------------------------------------------------------

MidiShare client application setup

---------------------------------------------------------------------------- */
Boolean SetUpMidi(void) {
	  if (!MidiShare()) {
		AlertUser("Can't access to 'MSHARE.DLL'");
		return false;
  }
  myRefNum = MidiOpen(AppliName);
  if ( myRefNum == MIDIerrSpace ) {
		AlertUser("Too much MidiShare client applications");
		return false;
  }
	MidiSetRcvAlarm(myRefNum,ReceiveEvents);
	MidiConnect(myRefNum,0,true);
	return true;
}


/* ----------------------------------------------------------------------------
			Alert messages
---------------------------------------------------------------------------- */
void AlertUser(LPCSTR mess)
{
	MessageBox (ghWnd, mess, AppliName,MB_OK|MB_ICONSTOP);
	SendMessage(ghWnd,WM_SYSCOMMAND,SC_CLOSE,0L);
}


/* ----------------------------------------------------------------------------

 Application receive alarm (called at interrupt level)
 msCtrl don't use incoming events

---------------------------------------------------------------------------- */
void FAR PASCAL _export ReceiveEvents(register short r) {
	MidiFlushEvs(r);
}


/* ----------------------------------------------------------------------------

 Private profile functions

---------------------------------------------------------------------------- */
void SaveState (StatePtr state)
{
	char buff[30];

	wsprintf (buff, "%d", dlgXPos);
	WriteState (sectionName, xEntryName, buff, profileName);
	wsprintf (buff, "%d", dlgYPos);
	WriteState (sectionName, yEntryName, buff, profileName);

	wsprintf (buff, "%d", (int)state[kPort]);
	WriteState (StateSectionName, portEntryName, buff, profileName);
	wsprintf (buff, "%d", (int)state[kChan]);
	WriteState (StateSectionName, chanEntryName, buff, profileName);
	wsprintf (buff, "%d", (int)state[kProgChge]);
	WriteState (StateSectionName, progChgeEntryName, buff, profileName);
	wsprintf (buff, "%d", (int)state[kVol]);
	WriteState (StateSectionName, volEntryName, buff, profileName);
	wsprintf (buff, "%d", (int)state[kModulation]);
	WriteState (StateSectionName, modulationEntryName, buff, profileName);
	wsprintf (buff, "%d", (int)state[kPan]);
	WriteState (StateSectionName, panEntryName, buff, profileName);
	wsprintf (buff, "%d", (int)state[kPitch]);
	WriteState (StateSectionName, pitchEntryName, buff, profileName);
	wsprintf (buff, "%d", (int)state[kVel]);
	WriteState (StateSectionName, velEntryName, buff, profileName);
}

//________________________________________________________________________
void LoadState (StatePtr state)
{
	int x, y, v;

	x = GetState (sectionName, xEntryName, kDefaultXPos, profileName);
	y = GetState (sectionName, yEntryName, kDefaultYPos, profileName);
	dlgXPos =  x ? x : kDefaultXPos;
	dlgYPos =  y ? y : kDefaultYPos;
	state[kPort]		= GetState (StateSectionName, portEntryName, 0, profileName);
	state[kChan]		= GetState (StateSectionName, chanEntryName, 0, profileName);
	state[kProgChge]	= GetState (StateSectionName, progChgeEntryName, 0, profileName);
	state[kVol]			= GetState (StateSectionName, volEntryName, 0, profileName);
	state[kModulation]= GetState (StateSectionName, modulationEntryName, 0, profileName);
	state[kPan]			= GetState (StateSectionName, panEntryName, 0, profileName);
	state[kPitch]		= GetState (StateSectionName, pitchEntryName, 0, profileName);
	state[kVel]			= GetState (StateSectionName, velEntryName, 0, profileName);
}

//________________________________________________________________________
BOOL FAR PASCAL _export AboutDlgProc(HWND hDlg,UINT message,
													UINT wParam,LONG lParam)
{
	char buff[10];

	switch (message) {
		case WM_INITDIALOG :
			abouthDlg = hDlg ;
			wsprintf(buff,"%d. %d",MidiGetVersion()/100,MidiGetVersion()%100);
			SetDlgItemText(hDlg, Version, buff);
			return TRUE ;

		case WM_COMMAND :
			switch(wParam) {
				case IDOK :
					EnableWindow(GetDlgItem(ghWnd,AboutButton),true);
					EndDialog(hDlg,0);
					abouthDlg = 0 ;
					break;
			}
			break;
	}
	return FALSE ;
}



